package edu.gatech.cercs.soa.core.fpq.impl;

import java.io.PrintWriter;
import java.util.*;

import org.xml.sax.Attributes;

import edu.gatech.cercs.soa.core.fpq.IItinerary;
import edu.gatech.cercs.soa.core.fpq.ITrip;
import edu.gatech.cercs.soa.core.xml.XMLable;


/**
 * Describe <code>ItineraryImpl</code> here.
 *
 * @author <a href="mailto:srihari@cc.gatech.edu">Hari Govindharaj</a>
 */
public class ItineraryImpl implements IItinerary, XMLable {

    private List<ITrip> trips;
    
    public ItineraryImpl() {
        trips = new ArrayList<ITrip>();
    }
    
    public void addTrip(ITrip trip) {
        trips.add(trip);
    }
    
    public ITrip createTrip() {
        return new TripImpl();
    }

    public List<ITrip> getTrips() {
        return trips;
    }

    public void removeAllTrips() {
        trips.clear();
    }

    public void removeTrip(ITrip trip) {
        trips.remove(trip);
    }

    ////////////////////////////////////////////////////////////////////////////
    // XMLable
    
    public void handleXMLCallback(String name, Attributes attrs,
            boolean isBeginElement, Map<String, Object> state) {
        if (isBeginElement) {
            if (name.equalsIgnoreCase("trip")) {
                state.put("trip", new TripImpl());               
            }
        }
        else {
            //Closing Tag
            if (name.equalsIgnoreCase("trip")) {
                trips.add((ITrip)state.remove("trip"));
            }
        }
        if (state.get("trip") != null) {
            ((XMLable)state.get("trip")).handleXMLCallback(name, attrs, 
                    isBeginElement, state);
        }
    }

    public void saveAsXML(PrintWriter writer) {
        StringBuffer sb = new StringBuffer();
        sb.append("<Itinerary>\n");
        writer.write(sb.toString());
        for (Iterator<ITrip> iter = trips.iterator(); iter.hasNext();) {
            ((XMLable)iter.next()).saveAsXML(writer);
        }
        sb.setLength(0);
        sb.append("</Itinerary>\n");
        writer.write(sb.toString());
    }

}
