package edu.gatech.cercs.soa.core.fpr.impl;

import java.io.PrintWriter;
import java.util.*;

import org.xml.sax.Attributes;

import edu.gatech.cercs.soa.core.fpr.*;
import edu.gatech.cercs.soa.core.xml.XMLable;


/**
 * Describe <code>FlightPricingResultImpl</code> here.
 *
 * @author <a href="mailto:srihari@cc.gatech.edu">Hari Govindharaj</a>
 */
public class FlightPricingResultImpl implements IFlightPricingResult, XMLable {

    private IHeaderResult hdr;
    private List<IItineraryResult> itineraries;
    
    public FlightPricingResultImpl() {
        hdr = new HeaderResultImpl();
        itineraries = new ArrayList<IItineraryResult>();
    }
    
    public void addItinerary(IItineraryResult it) {
        itineraries.add(it);
    }

    public IItineraryResult createItinerary() {
        return new ItineraryResultImpl();
    }
    
    public IHeaderResult getHeader() {
        if (hdr == null) {
            hdr = new HeaderResultImpl();
        }
        return hdr;
    }
    
    public List<IItineraryResult> getItineraries() {
        return itineraries;
    }

    public void removeItinerary(IItineraryResult it) {
        itineraries.remove(it);
    }
    
    public void setHeader(IHeaderResult hdr) {
        this.hdr = hdr;
    }

    ////////////////////////////////////////////////////////////////////////////
    // XMLable
    
    public void handleXMLCallback(String name, Attributes attrs,
            boolean isBeginElement, Map<String, Object> state) {
        if (isBeginElement) {
            if (name.equalsIgnoreCase("header")) {
                state.put("header", getHeader());
            }
            else if (name.equalsIgnoreCase("itinerary")) {
                state.put("itinerary", new ItineraryResultImpl());
            }
        }
        else {
            //Closing Tag
            if (name.equalsIgnoreCase("header")) {
                hdr = (IHeaderResult)state.remove("header");
            }
            else if (name.equalsIgnoreCase("itinerary")) {
                addItinerary((IItineraryResult)state.remove("itinerary"));
            }
        }
        
        XMLable hd = (XMLable)state.get("header");
        if (hd != null) {
            hd.handleXMLCallback(name, attrs, isBeginElement, state);
            return;
        }
        
        XMLable it = (XMLable)state.get("itinerary");
        if (it != null) {
            it.handleXMLCallback(name, attrs, isBeginElement, state);
        }
    }

    public void saveAsXML(PrintWriter writer) {
        writer.write("<FlightPricingResult>\n");
        ((XMLable)hdr).saveAsXML(writer);
        writer.write("<Itineraries>\n");
        for (Iterator<IItineraryResult> iter = itineraries.iterator(); iter.hasNext();) {
            ((XMLable)iter.next()).saveAsXML(writer);
        }
        writer.write("</Itineraries>\n");
        writer.write("</FlightPricingResult>\n");
    }

}