package edu.gatech.cercs.soa.core.fpr.impl;

import java.io.PrintWriter;
import java.util.*;

import org.xml.sax.Attributes;

import edu.gatech.cercs.soa.core.fpr.*;
import edu.gatech.cercs.soa.core.xml.XMLable;


/**
 * Describe <code>ItineraryResultImpl</code> here.
 *
 * @author <a href="mailto:srihari@cc.gatech.edu">Hari Govindharaj</a>
 */
public class ItineraryResultImpl implements IItineraryResult, XMLable {

    private IPriceResult price;
    private List<ITripResult> trips;
    
    public ItineraryResultImpl() {
        trips = new ArrayList<ITripResult>();
    }
    
    public void addTrip(ITripResult trip) {
        trips.add(trip);
    }
    
    public ITripResult createTrip() {
        return new TripResultImpl();
    }

    public IPriceResult getPrice() {
        if (price == null) {
            price = new PriceResultImpl();
        }
        return price;
    }

    public List<ITripResult> getTrips() {
        return trips;
    }

    public void removeTrip(ITripResult trip) {
        trips.remove(trip);
    }

    public void setPrice(IPriceResult price) {
        this.price = price;
    }

    ////////////////////////////////////////////////////////////////////////////
    // XMLable
    
    public void handleXMLCallback(String name, Attributes attrs,
            boolean isBeginElement, Map<String, Object> state) {
        if (isBeginElement) {
            if (name.equalsIgnoreCase("price")) {
                state.put("price", new PriceResultImpl());
            }
            else if (name.equalsIgnoreCase("trip")) {
                state.put("trip", new TripResultImpl());
            }
        }
        else {
            //Closing Tag
            if (name.equalsIgnoreCase("price")) {
                price = (IPriceResult)state.remove("price");
            }
            else if (name.equalsIgnoreCase("trip")) {
                addTrip((ITripResult)state.remove("trip"));
            }
        }
        XMLable price = (XMLable)state.get("price");
        if (price != null) {
            price.handleXMLCallback(name, attrs, isBeginElement, state);
        }
        XMLable trip = (XMLable)state.get("trip");
        if (trip != null) {
            trip.handleXMLCallback(name, attrs, isBeginElement, state);
        }
    }

    public void saveAsXML(PrintWriter writer) {
        writer.write("<Itinerary>\n");
        ((XMLable)getPrice()).saveAsXML(writer);
        for (Iterator<ITripResult> iter = trips.iterator(); iter.hasNext();) {
            ((XMLable)iter.next()).saveAsXML(writer);
        }
        writer.write("</Itinerary>\n");
    }

}
