package edu.gatech.cercs.soa.core.fpr.impl;

import java.io.PrintWriter;
import java.util.*;

import org.xml.sax.Attributes;

import edu.gatech.cercs.soa.core.fpr.IStopResult;
import edu.gatech.cercs.soa.core.fpr.ITripResult;
import edu.gatech.cercs.soa.core.xml.XMLable;


/**
 * Describe <code>TripResultImpl</code> here.
 *
 * @author <a href="mailto:srihari@cc.gatech.edu">Hari Govindharaj</a>
 */
public class TripResultImpl implements ITripResult, XMLable {

    private List<IStopResult> stops;
    
    public TripResultImpl() {
        stops = new ArrayList<IStopResult>();
    }
    
    public void addStop(IStopResult stop) {
        stops.add(stop);
    }
    
    public IStopResult createStop() {
        return new StopResultImpl();
    }

    public List<IStopResult> getStops() {
        return stops;
    }

    public void removeStop(IStopResult stop) {
        stops.remove(stop);
    }

    ////////////////////////////////////////////////////////////////////////////
    // XMLable
    
    public void handleXMLCallback(String name, Attributes attrs,
            boolean isBeginElement, Map<String, Object> state) {
        if (isBeginElement) {
            if (name.equalsIgnoreCase("stop")) {
                state.put("stop", new StopResultImpl());
            }
        }
        else {
            //Closing Tag
            if (name.equalsIgnoreCase("stop")) {
                addStop((IStopResult)state.remove("stop"));
            }
        }
        XMLable stop = (XMLable)state.get("stop");
        if (stop != null) {
            stop.handleXMLCallback(name, attrs, isBeginElement, state);
        }
    }

    public void saveAsXML(PrintWriter writer) {
        writer.write("<Trip>\n");
        for (Iterator<IStopResult> iter = stops.iterator(); iter.hasNext();) {
            ((XMLable)iter.next()).saveAsXML(writer);
        }
        writer.write("</Trip>\n");
    }

}
