package edu.gatech.cercs.soa.core.svc;

import java.rmi.Remote;
import java.rmi.RemoteException;


/**
 * The FlightPricingService is a service endpoint implemented by the GDS and is
 * invoked by the clients of GDS to search flight prices.
 * 
 * @author <a href="mailto:srihari@cc.gatech.edu">Hari Govindharaj</a>
 */
public interface FlightPricingService extends Remote {

    /**
     * The searchPrices call queues the query in the GDS system for further
     * processing.
     * 
     * @param query - non-null string in XML format
     * @return String - contains search status info in XML format
     * @throws RemoteException
     */
    public String searchPrices(String query) throws RemoteException;

    /**
     * Poll the flight pricing state table of GDS for the search prices result.
     * 
     * @param queryId - non-null string
     * @return String - contains search status (or) result in XML format
     * @throws RemoteException
     */
    public String retrievePrices(String customerId, String queryId) 
            throws RemoteException;
    
}
