package edu.gatech.cercs.soa.core.xml;

import java.util.*;

import org.xml.sax.*;
import org.xml.sax.helpers.*;



/**
 * Describe <code>XMLHandlerBase</code> here.
 */
public class XMLHandlerBase extends DefaultHandler {
    
    protected Object helper;
    protected Map<String, Object> state;
    protected AttributesImpl attrs;
    protected XMLable xmlObj;
    protected StringBuffer chars;

    public XMLHandlerBase(Object helper) {
        attrs = new AttributesImpl();
        state = new HashMap<String, Object>();
        chars = new StringBuffer();
        setHelper(helper);
    }

    /**
     * The helper object is passed on to the callback handler, and winds up in
     * the XMLable state hashtable under the key 'helper'.
     * 
     * @param helper
     *            the helper object.
     */
    protected void setHelper(Object helper) {
        this.helper = helper;
        if (helper != null)
            state.put("helper", helper);
    }

    ////////////////////////////////////////////////////////////////////////////
    // DTDHandler interface
    
    public void notationDecl(String name, String publicId, String systemId) { 
        System.err.println("notationDecl: " + name + " pub: " + publicId
                + " sys: " + systemId);
    }   
    
    public void unparsedEntityDecl(String name, String publicId, 
            String systemId, String notationName) {
        System.err.println("unparsedEntityDecl: " + name + " pub: " + publicId
                + " sys: " + systemId);
    }

    ////////////////////////////////////////////////////////////////////////////
    // ContentHandler interface
    
    public void startElement(String uri, String localName, String qName, 
            Attributes attributes) throws SAXException {
    }

    public void endElement(String uri, String localName, String qName)
            throws SAXException {
        attrs.clear();
        if (chars.length() > 0) {
            while (chars.length() > 0
                    && Character.isWhitespace(chars.charAt(chars.length() - 1)))
                chars.deleteCharAt(chars.length() - 1);
            while (chars.length() > 0
                    && Character.isWhitespace(chars.charAt(0)))
                chars.deleteCharAt(0);
            if (chars.length() > 0) {
                attrs.addAttribute(uri, localName, "characters", "chars", chars.toString());
                //startElement(uri, "characters", qName, attrs); //Why?
                chars.delete(0, chars.length());
            }
        }
    }

    public void characters(char ch[], int start, int length) {
        if (length == 0) {
            return;
        }
        chars.append(ch, start, length);
    }
    
    public void ignorableWhitespace (char ch[], int start, int length) {
        String s = new String(ch, start, length);
        System.err.println("Ignorable white space:" + s + ":");
    }


    public void processingInstruction (String target, String data) {
        System.err.println("processing instruction: " + target + " data: "
                + data);
    }
    
    ////////////////////////////////////////////////////////////////////////////
    // ErrorHandler interface
    
    public void warning(SAXParseException e) {
        System.err.println("sax parsing warning: " + e);
        e.printStackTrace();
    }

    public void error(SAXParseException e) {
        System.err.println("sax parsing error: " + e);
        e.printStackTrace();
    }

    public void fatalError(SAXParseException e) {
        System.err.println("sax fatal parsing error: " + e);
        e.printStackTrace();
    }

}
