package edu.gatech.cercs.soa.core.xml;

import java.io.IOException;
import java.io.Reader;
import java.util.*;

import javax.xml.parsers.*;

import org.xml.sax.*;


/**
 * Takes an input stream and an XMLable, and forwards XML callbacks.
 * This should be used by empty XMLable object to start the parsing process.
 */
public class XMLProxy {

    private XMLable xmlObj;
    private Map<String, Object> env;

    public XMLProxy(XMLable xmlObj) {
        this.xmlObj = xmlObj;
    }

    public XMLProxy(XMLable xmlObj, Map<String, Object> env) {
        this.xmlObj = xmlObj;
        this.env = env;
    }

    public void parse(Reader reader) throws IOException {
        ProxyHandler ph = new ProxyHandler(xmlObj, env);
        try {
            SAXParserFactory factory = SAXParserFactory.newInstance();
            SAXParser saxParser = factory.newSAXParser();
            InputSource input = new InputSource(reader);
            saxParser.parse(input, ph);
        } 
        catch (SAXException e) {
            e.printStackTrace();
            throw new IOException(e.getMessage());
        } 
        catch(ParserConfigurationException pce) {
            pce.printStackTrace();
            throw new IOException(pce.getMessage());
        }
    }

}


class ProxyHandler extends XMLHandlerBase {

    private XMLable xmlObj;
    private Map<String, Object> state;

    public ProxyHandler(Object xmlObj) {
        this(xmlObj, new HashMap<String, Object>());
    }

    public ProxyHandler(Object xmlObj, Map<String, Object> env) {
        super(null);
        this.xmlObj = (XMLable)xmlObj;
        this.state = env;
        if (this.state == null) {
            this.state = new HashMap<String, Object>();
        }
    }

    public void startElement(String uri, String localName, String qName, 
            Attributes attrs) throws SAXException {
        try {
            super.startElement(uri, localName, qName, attrs);
            xmlObj.handleXMLCallback(qName, attrs, true, state);
        } 
        catch (Exception e) {
            e.printStackTrace();
            throw new SAXException(e.getMessage());
        }
    }

    public void endElement(String uri, String localName, String qName)
            throws SAXException {
        try {
            super.endElement(uri, localName, qName);
            xmlObj.handleXMLCallback(qName, attrs, false, state);
        } 
        catch (Exception e) {
            e.printStackTrace();
            throw new SAXException(e.getMessage());
        }
    }
}
