package edu.gatech.cercs.soa.core.xml;

import java.io.PrintWriter;
import java.util.Map;

import org.xml.sax.Attributes;


/**
 * An interface an object must implement to save itself as XML, and to
 * restore itself from XML.
 */
public interface XMLable {

    /**
     * A special attribute that is passed to the handler callback via
     * the state hashtable. Its value is an object that is needed to
     * initialize this object.
     */
    public static final String HELPER_KEY = "helper";


    /**
     * Writes this object as an XML string.
     *
     * @param writer the stream into which to write XML.
     */
    public void saveAsXML(PrintWriter writer);

    /**
     * Handles SAX callbacks.
     *
     * @param name the name of the tag
     * @param attrs the attribute list, or null if the end tag.
     * @param isTagStart true if this is the beginElement callback,
     *           false if this is the endElement callback.
     */
    public void handleXMLCallback(String name, Attributes attrs, 
            boolean isBeginElement, Map<String, Object> state);

}
