package edu.gatech.cercs.soa.gds.client;


import java.io.*;
import java.net.URL;

import javax.xml.namespace.QName;
import javax.xml.rpc.Service;
import javax.xml.rpc.ServiceFactory;

import edu.gatech.cercs.soa.core.FlightPricingUtils;
import edu.gatech.cercs.soa.core.PropertyLoader;
import edu.gatech.cercs.soa.core.fpq.IHeader;
import edu.gatech.cercs.soa.core.fpr.IHeaderResult;
import edu.gatech.cercs.soa.gds.web.FlightPricingService;

/**
 * Describe <code>FlightPricingServiceClient</code> here.
 * 
 * @author <a href="mailto:srihari@cc.gatech.edu">Hari Govindharaj</a>
 */
public class FlightPricingServiceClient {

   
   private static final String SERVICE_URL = "service.URL";
   private static final String SERVICE_NAMESPACE_URI = "service.NamespaceURI";
   private static final String SERVICE_NAME = "service.Name";
   
   private static final String PROPERTIES_FILE = "client.properties";

   private PropertyLoader propLoader;
   private String queryFileName;

   public FlightPricingServiceClient(String queryFileName) {
      this.queryFileName = queryFileName;
      this.propLoader = PropertyLoader.getInstance(PROPERTIES_FILE);
   }
   
   
   public static void main(String[] args) throws Exception {
      FlightPricingServiceClient fpClient = new FlightPricingServiceClient(args[0]);
      fpClient.send();
   }

   public void send() throws Exception {
      File f = new File(queryFileName);
      if (!f.exists() || !f.isFile()) {
         System.out.println(queryFileName + "is not a valid file name!");
         return;
      }

      BufferedReader br = new BufferedReader(new FileReader(f));
      StringBuffer sb = new StringBuffer();
      String line = null;
      while ((line = br.readLine()) != null) {
         sb.append(line);
      }
      br.close();

      String query = sb.toString();
      IHeader h = FlightPricingUtils.parseQueryHeader(query);

      URL url = new URL(propLoader.getValue(SERVICE_URL));

      QName qname = new QName(propLoader.getValue(SERVICE_NAMESPACE_URI), 
               propLoader.getValue(SERVICE_NAME));
      ServiceFactory factory = ServiceFactory.newInstance();
      Service service = factory.createService(url, qname);
      FlightPricingService fpService = (FlightPricingService) service
               .getPort(FlightPricingService.class);

      String result = fpService.searchPrices(query);
      IHeaderResult hr = FlightPricingUtils.parseResultHeader(result);
      while (!IHeaderResult.STATUS_COMPLETE.equalsIgnoreCase(hr.getStatus())
               && !IHeaderResult.STATUS_ERROR.equalsIgnoreCase(hr.getStatus())) {
         System.out.println("Query Status: " + hr.getStatusDetail());
         System.out.println("Result: " + result);
         Thread.sleep(500);
         result = fpService.retrieveResult(h.getCustomerId(), h.getQueryId());
         hr = FlightPricingUtils.parseResultHeader(result);
      }

      System.out.println("Flight Pricing Service Result: " + result);
   }
}
