package edu.gatech.cercs.soa.gds.ejb;


import java.net.URL;

import javax.ejb.*;
import javax.jms.*;
import javax.xml.namespace.QName;
import javax.xml.rpc.Service;
import javax.xml.rpc.ServiceFactory;

import edu.gatech.cercs.soa.core.FlightPricingUtils;
import edu.gatech.cercs.soa.core.fpr.IHeaderResult;
import edu.gatech.cercs.soa.gds.FlightPricingState;
import edu.gatech.cercs.soa.tws.web.TravelWebService;

/**
 * 
 * @ejb.bean name = "FlightPricingResultStateMDB" display-name =
 *           "FlightPricingResultStateMDB" description = "Flight Pricing Result
 *           State Message Driven Bean" destination-type = "javax.jms.Queue"
 * 
 */
public class FlightPricingResultStateMDB implements MessageDrivenBean,
        MessageListener {

    private static final long serialVersionUID = 1L;

    /** The MessageDrivenContext */
    // private MessageDrivenContext context;
    /**
     * An <code>ejbCreate</code> method as required by the EJB specification.
     * <br>
     * 
     * The container calls the instances <code>ejbCreate</code> method
     * immediately after instantiation.
     * 
     * @ejb.create-method
     */
    public void ejbCreate() {
    }

    /**
     * The <code>ejbRemove()</code> method as required by the EJB
     * specification.<br>
     * 
     * A container invokes this method before it ends the life of the object.
     * This happens as a result of a client's invoking a remove operation, or
     * when a container decides to terminate the object after a timeout. <br>
     * 
     * This method is called with no transaction context.
     * 
     * @throws EJBException
     *             Thrown if the instance could not perform the function
     *             requested by the container because of an system-level error.
     */
    public void ejbRemove() throws EJBException {
    }

    /**
     * Set the associated context. The container calls this method after the
     * instance creation. <br>
     * 
     * The enterprise bean instance should store the reference to the context
     * object in an instance variable. <br>
     * 
     * This method is called with no transaction context.
     * 
     * @param newContext
     *            A MessageDrivenContext interface for the instance.
     * 
     * @throws EJBException
     *             Thrown by the method to indicate a failure caused by a
     *             system-level error.
     */
    public void setMessageDrivenContext(MessageDrivenContext newContext)
            throws EJBException {
        // context = newContext;
    }

    /**
     * Process a message. <br>
     * 
     * @param message
     *            The message to process.
     */
    public void onMessage(Message message) {
        TextMessage textMessage = (TextMessage) message;
        try {
            String request = textMessage.getText();
            // System.out.println("Result Recieved \n"+request);

            send(request);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void send(String result) throws Exception {
        //Parse the header of the result
        IHeaderResult hdr = FlightPricingUtils.parseResultHeader(result);
        if (hdr == null) {
            return; //throw an error?
        }
        
        //Check if the query mode is async or polling
        if (IHeaderResult.QUERYMODE_ASYNC.equalsIgnoreCase(hdr.getQueryMode())) {
           //Async query mode, post the result back to the client service
           //For now - post back results only to the TWS 
           //TODO - remove the TravelWebService interface dependency
           //TODO - avoid overloading customer id and service endpoint
           String customerId = hdr.getCustomerId();
           String[] endpoint = customerId.split("::");
           if (endpoint != null && endpoint.length == 4) {
              URL url = new URL(endpoint[0]);
              QName qname = new QName(endpoint[1], endpoint[2]);
              ServiceFactory factory = ServiceFactory.newInstance();
              Service service = factory.createService(url, qname);
              TravelWebService twService = (TravelWebService)service
                       .getPort(TravelWebService.class);
              twService.updateFlightPrices(result);
           }           
        }
        else {
            //Sync query mode, store the result in flight pricing state table
            FlightPricingState.getInstance().updateResult(result);
        }
    }
}
